module Permutation
  class Config
    class << self
      ACCOUNT_CONFIG_FILE    = File.join(Etc.getpwuid.dir, '.permutation')
      REPOSITORY_CONFIG_FILE = File.join(Dir.pwd, '.permutation')

      def account
        Permutation::Config.read(ACCOUNT_CONFIG_FILE) || init
      end

      def init
        access_token = nil
        json = {}

        puts "Log In to Permutation"
        email  = ask("Email: ")
        password = ask("Password: ") {|q| q.echo = false}

        response = Permutation::API.post '/api/session.json', { email: email, password: password }

        if response.status == 201
          session = response.body.slice("email","access_token")
          json.merge! session
          json.merge! host: Permutation::API.host unless Permutation::API.host == Permutation::API::DEFAULT_HOST

          write(json, ACCOUNT_CONFIG_FILE)
          json
        else
          puts
          puts "Invalid login credentials"
          exit
        end
      end

      def host
        if config = Permutation::Config.read(ACCOUNT_CONFIG_FILE)
          config['host']
        else
          nil
        end
      end
    end


    class << self
      def write(json, file)
        puts "Writing to #{file}" if DEBUG

        _file = File.open( file, "w+" )
        _file.write(JSON.pretty_generate(json))
        _file.close
      end

      def read(file)
        return JSON.parse( IO.read(file), symbolize_names: false )
      rescue
        nil
      end

      def version
        read["version"]
      end

      private

      def not_setup!
        puts "Permutation is not setup, please run 'permutation init'"
        exit 1
      end
    end
  end
end
