require 'open-uri'
require 'sinatra'
require 'byebug'

module Permutation
  class Server < Sinatra::Application
    set :static, false
    set :bind, '0.0.0.0'

    after do
      if response.headers["Content-Type"] &&
         response.headers["Content-Type"].include?("text/html")

        script_host = ENV['SCRIPT_HOST'] || "http://permutation-cdn.s3.amazonaws.com"

        response.body.each do |part|
          insert_at = part.index('<head>') + 6
          unless insert_at.nil?
            part.insert(insert_at, %%
<script type='text/javascript' src='#{script_host}/permutation.js'></script>
<script type='text/javascript'>
  $P.init();
</script>
          %)
          end
        end

        new_size = 0
        response.body.each{|part| new_size += part.bytesize}
        response.headers.merge!("Content-Length" => new_size.to_s)
      end
    end

    get '/api/*' do
      host = ENV['API_HOST'] || "http://router.permutation.io"
      url  = "/api/"
      if splat = params.delete('splat')
        url += splat.join
      end

      conn = Faraday.new(url: host) do |faraday|
        faraday.request  :url_encoded
        faraday.response :logger
        faraday.adapter  Faraday.default_adapter
      end

      res = conn.get url
      status res.status
      res.body
    end

    post '/api/*' do
      host = ENV['API_HOST'] || "http://router.permutation.io"
      url  = "/api/"
      if splat = params.delete('splat')
        url += splat.join
      end

      conn = Faraday.new(url: host) do |faraday|
        faraday.request  :url_encoded
        # faraday.response :logger
        faraday.adapter  Faraday.default_adapter
      end

      res = conn.post url, params
      status res.status
      res.body
    end

    get '*' do
      uri = params[:splat].first
      uri = '/index.html' if uri =~ /^\/$/

      f = File.join('.', uri)
      if File.directory?(f)
        uri += "/index.html"
        f = File.join('.', uri)
      end

      if File.exists?(f) && File.file?(f)
        if uri =~ /\.html$/
          response.headers['Content-Type'] = `file --mime -b #{f}`
          File.read(f)
        else
          send_file f
        end
      else
        status 404
        "Not Found"
      end
    end

    post    ('*') { catchall }
    put     ('*') { catchall }
    patch   ('*') { catchall }
    delete  ('*') { catchall }
    options ('*') { catchall }

    private

    def catchall

      params.delete 'splat'
      params.delete 'captures'

      puts
      puts "Conversion!"
      puts "  params: #{params.inspect}"
      puts

      status 201
    end
  end
end
